<?php
/*  latex2img uses latexrender to turn a LaTeX expression into an image for use in HTML pages    
    based on a design and code originally from Georg Sorst, 2004, which powers matheboard.de

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    Usage:
    http://path/to/pub/latexrender/scripts/latex2img.php?x?mode=y[&debug]

    where x is a valid latex expression and mode is one of the supported
    latexrender classes (automatically set by the LatexRender PmWiki plug-in), 
    math, displaymath, equation, align, alignstar, subequations

    Output will be image/png.

    Requirements:
    - LatexRender installed and working with PmWiki on the server to test markup
    - Using PmWiki+LatexRender recipe on requesting server will insert valid modes

*/
$latexrender_path = str_replace('/scripts','',dirname(__FILE__));

$UrlScheme = (@$_SERVER['HTTPS']=='on' || @$_SERVER['SERVER_PORT']==443)
             ? 'https' : 'http';
$ScriptUrl = $UrlScheme.'://'.$_SERVER['HTTP_HOST'].$_SERVER['SCRIPT_NAME'];
$latexrender_path_http = preg_replace('#/scripts/[^/]*$#', '', $ScriptUrl, 1);

$chars = 0;
$errors = array();
$ret = 0;
$trans_tbl = array(
    '%B'    => '^',
    '%20'     => ' ',
    '%3E'     => '>',
    '%3C'     => '<',
    '%E4'     => 'ä',
    '%F6'     => 'ö',
    '%FC'     => 'ü',
    '%C4'     => 'Ä',
    '%D6'     => 'Ö',
    '%DC'     => 'Ü',
    '%DF'     => 'ß',
    '%25'     => '\%',

    // additional translations for Opera (is this UTF8?)
    '%C3%A4'    => 'ä',
    '%C3%B6'    => 'ö',
    '%C3%BC'    => 'ü',
    '%C3%9F'    => 'ß',
    '%C3%84'    => 'Ä',
    '%C3%96'    => 'Ö',
    '%C3%9C'    => 'Ü',
    '%C2%B'     => '^',
    '%22'       => '"',
    '%27'       => '\'',
    '%5C'       => '\\',
    
    // escape any remaining %, we don't want comments
    '%'        => '\%',
        
        // additional translations for Safari
        '%7B' => '{',
        '%7D' => '}',
        '%7C' => '|',
        '%5E' => '^',
    '%5D' => ']',
    '%5B' => '[',

    // html linebreak to newline
    '%3Cbr%20/%3E' => '\\\\',
);

function register_error($message) {
    global $chars, $errors, $ret;    
    // save error messages in $errors
    $ret = 1;
    $errors[] = $message;
    if (strlen($message) > $chars)
        $chars = strlen($message);
}

function error_formula($errors) {
    if(empty($errors)) register_error('Unknown error, no image generated');
    return '\begin{tabular}{ |l| }\hline '.implode("\\\\ ",$errors).' \\\\ \hline \end{tabular}';
}


function LatexRServer($class, $text, $svg) {
// -------------------------------------------------------------------------
// adjust this to match your system configuration
    global $latexrender_path, $latexrender_path_http, $LatexrenderLatexPath, $LatexrenderDvipsPath,
        $LatexrenderDvisvgmPath, $LatexrenderConvertPath, $LatexrenderIdentifyPath, $LatexrenderImageFmt;
    $LatexrenderImageFmt     =  $svg ? "svg" : "png";
//  typical linux binary locations 
    $LatexrenderLatexPath    = "/usr/bin/latex";
    $LatexrenderDvipsPath    = "/usr/bin/dvips";
    $LatexrenderDvisvgmPath  = "/usr/bin/dvisvgm";
    $LatexrenderConvertPath  = "/usr/bin/convert";
    $LatexrenderIdentifyPath = "/usr/bin/identify";
//  typical mac os x binary locations
/*
    $LatexrenderLatexPath    = "/usr/texbin/latex";
    $LatexrenderDvipsPath    = "/usr/texbin/dvips";
    $LatexrenderDvisvgmPath  = "/usr/texbin/dvisvgm";
    $LatexrenderConvertPath  = "/opt/local/bin/convert";
    $LatexrenderIdentifyPath = "/opt/local/bin/identify";
*/
// -------------------------------------------------------------------------
// include main rendering class, note that it must be in the scripts directory
    include_once($latexrender_path."/scripts/class.latexrender.php");
    
    $latex = new LatexRender($latexrender_path."/pictures",
        $latexrender_path_http."/pictures", $latexrender_path."/tmp");

//  set location of required binaries
    $latex->_latex_path = $LatexrenderLatexPath;
    $latex->_dvips_path = $LatexrenderDvipsPath;
    $latex->_dvisvgm_path  = $LatexrenderDvisvgmPath;
    $latex->_convert_path  = $LatexrenderConvertPath;
    $latex->_identify_path = $LatexrenderIdentifyPath;
    $latex_formula = trim($text);
    $latex->_image_format = $LatexrenderImageFmt;
    $url = $latex->getFormulaURL($latex_formula,$class);

   if (!$url) { 
       register_error("Unparseable or potentially dangerous latex formula");
       register_error("Error $latex->_errorcode $latex->_errorextra" );
   }
             
    return $url;
}


$query = $_SERVER['QUERY_STRING'];
//$query = stripslashes($query);
$query = strtr($query, $trans_tbl);

// ?mode=... appended?
$pos = strrpos($query, '?');

// $pos === true appears to not be the same as $pos !== false
$svg = false;
$debug = false;
if (($pos !== false) && (substr($query, $pos+1, 5) == 'mode=')) {
    $mode = substr($query, $pos+6);
    
    // if $mode is like ?mode=...&debug or just ?mode=&debug switch to debug mode
    if (strcmp(substr($mode, -6), '&debug') == 0) {
        $debug = true;
        $mode = substr($mode, 0, strlen($mode) - 6);
    } elseif (strcmp(substr($mode, -4), '&svg') == 0) {
        $svg = true;
        $mode = substr($mode, 0, strlen($mode) - 4);
    }
    // if there's a ?mode=..., use $query only upto the last '?'
    $f = substr($query, 0, $pos);
} else {
    // if there's a '?', but no immediately following 'mode=...' use full $query
    $f = $query;
}

$url = LatexRServer($mode, $f, $svg);

if (!$debug) {
    date_default_timezone_set('Pacific/Auckland');
    header('Content-Type: image/'.($svg ? 'svg+xml' : 'png'));
    header('Expires: '.date('r', time()+60*60*24*7));
    if ($ret) $url = LatexRServer('displaymath', error_formula($errors), $svg);
    readfile($url);
} else {
    echo "Function: $f<br />";
     echo "Mode: $mode<br />";
    echo 'Errors: <br />';
    if ($ret == 0) echo 'None (url: '.$url.')';
    else { echo '<pre>'; print_r($errors); echo '</pre>'; }
}

?>
